class Dandy{
  
  PVector loc;
  PVector vel;
  PVector acc;
  float r;
  float maxforce;    // Maximum steering force
  float maxspeed;    // Maximum speed
  
  float sizemodx;
  float sizemody;
  boolean at_init;
  PVector init_loc;
  float init_angle;
  PImage dandy;
  
  Dandy(PVector initial_loc, float angle, int size_mod, float ms, float mf ){
    loc = initial_loc.get();
    vel = new PVector(0,0);
    acc = new PVector(0,0);
    maxforce = mf;
    maxspeed = ms;
    
    dandy = loadImage("dlion.png");
    sizemodx = dandy.width*(size_mod*0.14);
    sizemody = dandy.height*(size_mod*0.14);
    r = sizemody;
    
    at_init = true;
    init_loc = initial_loc;
    init_angle = angle;
  }
  
  void wind(){
    if(at_init == true){
      at_init = false;
      vel = new PVector(cos(init_angle), sin(init_angle));
    }
    else{
      at_init = true;
      vel = new PVector(0,0);
      loc = init_loc;
    }
  }
  
  void gravity(){
    acc.add(steer(new PVector(0.5*loc.x, 0.5*height-r), true));
  }
 
  void run(ArrayList dandys, PVector wind[]) {
    if(at_init == false){
      follow(wind);
      if(abs(vel.mag()) >= 0 && loc.y <= height-sizemody){
        acc.add(separate(dandys));
      }
      gravity();
      
      update();
      borders();
      render(true);
    }
    
    else{    
      update();
      borders();
      render(false);
      
      PVector this_wind = new PVector(0,0);
      int incr = (int)(width/32);
      int x = (int)(loc.x);
      for(int i = 0; i < 32; i++){
        if(x <= i*28){
          this_wind = wind[i].get();
          break;
        }
      }
      if(abs(this_wind.mag()) >= 90){
        wind();
      }  
    }
  }
  
  // Method to update location
  void update() {
    // Update velocity
    vel.add(acc);
    // Limit speed
    vel.limit(maxspeed);
    
    loc.add(vel);
    // Reset accelertion to 0 each cycle
    acc.mult(0);
  }
 
  void follow(PVector wind[]) {
    // Look ahead
    PVector ahead = vel.get();
    ahead.normalize();
    ahead.mult(28); 
    PVector lookup = PVector.add(loc,ahead);

    
    // What is the vector at that spot in the flow field?
    PVector desired = new PVector(0,0);
    int incr = (int)(width/32);
    int x = (int)(lookup.x);
    for(int i = 0; i < 33; i++){
      if(x <= i*28){
        if(i <= 28){
          desired = wind[1].get();
          break;
        }
        if(i == 32){
          desired = wind[31].get();
          break;
        }
        desired = wind[i].get();
        break;
      }
    }
    
    
    // Scale it up by maxspeed
    desired.mult(maxspeed);
    // Steering is desired minus velocity
    PVector steer = PVector.sub(desired, vel);
    steer.limit(maxforce);  // Limit to maximum steering force
    
    acc.add(steer);
    
  } 
  
  
  // A method that calculates a steering vector towards a target
  // Takes a second argument, if true, it slows down as it approaches the target
  PVector steer(PVector target, boolean slowdown) {
    PVector steer;  // The steering vector
    PVector desired = PVector.sub(target,loc);  // A vector pointing from the location to the target
    float d = desired.mag(); // Distance from the target is the magnitude of the vector
    // If the distance is greater than 0, calc steering (otherwise return zero vector)
    if (d > 0) {
      // Normalize desired
      desired.normalize();
      // Two options for desired vector magnitude (1 -- based on distance, 2 -- maxspeed)
      if ((slowdown) && (d < 100.0f)) desired.mult(maxspeed*(d/100.0f)); // This damping is somewhat arbitrary
      else desired.mult(maxspeed);
      // Steering = Desired minus Velocity
      steer = PVector.sub(desired,vel);
      steer.limit(maxforce);  // Limit to maximum steering force
    } else {
      steer = new PVector(0,0);
    }
    return steer;
  }
  
  void render(boolean is_init) {
    // Display dandy image the direction of velocity
    if(at_init == false){
      pushMatrix();
      translate(loc.x,loc.y);
      rotate(PI);
      image(dandy, 0, 0, sizemodx, sizemody);
      popMatrix();
    }
    else{
      pushMatrix();
      translate(loc.x, loc.y);
      rotate(init_angle+radians(270));
      translate(0-(sizemodx/2), 0);
      image(dandy, 0, 0, sizemodx, sizemody);
      popMatrix();
    }
  }
  
  // Wraparound
  void borders() {
    if (loc.x < -r) loc.x = width+r;
    if (loc.x > width+r) loc.x = -r;
    if (loc.y > height){
      vel = new PVector(0,0);
      loc.y = height;
    }
  }
  
    // Separation
  PVector separate (ArrayList dandys) {
    float desiredseparation = r;
    PVector steer = new PVector(0,0,0);
    int count = 0;
    // For every Dandy in the system, check if it's too close
    for (int i = 0 ; i < dandys.size(); i++) {
      Dandy other = (Dandy) dandys.get(i);
      float d = PVector.dist(loc,other.loc);
      // If the distance is greater than 0 and less than an arbitrary amount (0 when you are yourself)
      if ((d > 0) && (d < desiredseparation)) {
        // Calculate vector pointing away from neighbor
        PVector diff = PVector.sub(loc,other.loc);
        diff.normalize();
        diff.div(d);        // Weight by distance
        steer.add(diff);
        count++;            // Keep track of how many
      }
    }
    // Average -- divide by how many
    if (count > 0) {
      steer.div((float)count);
    }

    // As long as the vector is greater than 0
    if (steer.mag() > 0) {
      // Implement Reynolds: Steering = Desired - Velocity
      steer.normalize();
      steer.mult(maxspeed);
      steer.sub(vel);
      steer.limit(maxforce);
    }
    return steer;
  }
  
  boolean at_init(){
    return at_init;
  }
}
