import processing.core.*; 
import processing.xml.*; 

import krister.Ess.*; 

import krister.Ess.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class proj3 extends PApplet {



ArrayList dandys;
Stem stem;

/* Audio Set-up */
int bufferSize;
int steps;
float limitDiff;
int numAverages=32;
float myDamp=.1f;
float maxLimit,minLimit;
FFT myFFT;
AudioInput myInput;
PVector[] wind;

public void setup(){
  size(900,600);
  smooth();
  
  dandys = new ArrayList();
  
  float angle;
  float x;
  float y;
  for(float i= 0; i < 126; i ++){
    angle = (float)i/126*TWO_PI;
    x = width/2 + 26*cos(angle);
    y = height/2 + 26*sin(angle);
    dandys.add(new Dandy(new PVector(x,y), angle, ((int)i)%7, 5.0f, 1.0f));
  }
  
  stem = new Stem();
  wind = new PVector[32];
  
  
  // start up Ess
  Ess.start(this);  

  // set up our AudioInput
  bufferSize=32;
  myInput=new AudioInput(bufferSize);

  // set up our FFT
  myFFT=new FFT(bufferSize*2);
  myFFT.equalizer(true);

  // set up our FFT normalization/dampening
  minLimit=.005f;
  maxLimit=.05f;
  myFFT.limits(minLimit,maxLimit);
  myFFT.damp(myDamp);
  myFFT.averages(numAverages);

  // get the number of bins per average 
  steps=bufferSize/numAverages;

  // get the distance of travel between minimum and maximum limits
  limitDiff=maxLimit-minLimit;

  myInput.start();
  frameRate(45);
}

public void draw(){
  background(0xff97C9FF);
  
  fill(0xff8DBFFA);
  /* Draw audio bars in background*/
  for (int i=0; i<32; i++) {
     if(i == 0){
       rect(28*i,height,28,-(myFFT.spectrum[1]*1000));
       continue;
     }
     if(i == 31){       
       rect(28*i,height,32,-(myFFT.spectrum[1]*1000));
       break;
     }
       
     rect(28*i,height,28,-(myFFT.spectrum[i]*1000));
  }
  
  stem.display();
  updateWind();
  for(int i=0; i < 126; i++){
    Dandy index = (Dandy)dandys.get(i);
    index.run(dandys, wind);
  }
}
  
// Make a new flowfield
 public void keyPressed() {
  for(int i=0; i < 126; i++){
    Dandy index = (Dandy)dandys.get(i);
    
    if(index.at_init()){
      break;
    }
    index.wind();
  }
}

public void audioInputData(AudioInput theInput) {
  myFFT.getSpectrum(myInput);
}

public void updateWind() {
  float sizemod = 0;
  float theta = 0;

  for (int i = 0; i < 32; i++) {
    // Use perlin noise to get an angle between 0 and 2 PI
    theta = map(myFFT.spectrum[i],.005f,.05f,0,TWO_PI);
    
    // Generate length of the vector to indicate the level of sound
    sizemod = 1000*myFFT.spectrum[i];
    
    // Polar to cartesian coordinate transformation to get x and y components of the vector
    wind[i] = new PVector(sizemod*cos(theta),sizemod*sin(theta));
  }
}

class Dandy{
  
  PVector loc;
  PVector vel;
  PVector acc;
  float r;
  float maxforce;    // Maximum steering force
  float maxspeed;    // Maximum speed
  
  float sizemodx;
  float sizemody;
  boolean at_init;
  PVector init_loc;
  float init_angle;
  PImage dandy;
  
  Dandy(PVector initial_loc, float angle, int size_mod, float ms, float mf ){
    loc = initial_loc.get();
    vel = new PVector(0,0);
    acc = new PVector(0,0);
    maxforce = mf;
    maxspeed = ms;
    
    dandy = loadImage("dlion.png");
    sizemodx = dandy.width*(size_mod*0.14f);
    sizemody = dandy.height*(size_mod*0.14f);
    r = sizemody;
    
    at_init = true;
    init_loc = initial_loc;
    init_angle = angle;
  }
  
  public void wind(){
    if(at_init == true){
      at_init = false;
      vel = new PVector(cos(init_angle), sin(init_angle));
    }
    else{
      at_init = true;
      vel = new PVector(0,0);
      loc = init_loc;
    }
  }
  
  public void gravity(){
    acc.add(steer(new PVector(0.5f*loc.x, 0.5f*height-r), true));
  }
 
  public void run(ArrayList dandys, PVector wind[]) {
    if(at_init == false){
      follow(wind);
      if(abs(vel.mag()) >= 0 && loc.y <= height-sizemody){
        acc.add(separate(dandys));
      }
      gravity();
      
      update();
      borders();
      render(true);
    }
    
    else{    
      update();
      borders();
      render(false);
      
      PVector this_wind = new PVector(0,0);
      int incr = (int)(width/32);
      int x = (int)(loc.x);
      for(int i = 0; i < 32; i++){
        if(x <= i*28){
          this_wind = wind[i].get();
          break;
        }
      }
      if(abs(this_wind.mag()) >= 90){
        wind();
      }  
    }
  }
  
  // Method to update location
  public void update() {
    // Update velocity
    vel.add(acc);
    // Limit speed
    vel.limit(maxspeed);
    
    loc.add(vel);
    // Reset accelertion to 0 each cycle
    acc.mult(0);
  }
 
  public void follow(PVector wind[]) {
    // Look ahead
    PVector ahead = vel.get();
    ahead.normalize();
    ahead.mult(28); 
    PVector lookup = PVector.add(loc,ahead);

    
    // What is the vector at that spot in the flow field?
    PVector desired = new PVector(0,0);
    int incr = (int)(width/32);
    int x = (int)(lookup.x);
    for(int i = 0; i < 33; i++){
      if(x <= i*28){
        if(i <= 28){
          desired = wind[1].get();
          break;
        }
        if(i == 32){
          desired = wind[31].get();
          break;
        }
        desired = wind[i].get();
        break;
      }
    }
    
    
    // Scale it up by maxspeed
    desired.mult(maxspeed);
    // Steering is desired minus velocity
    PVector steer = PVector.sub(desired, vel);
    steer.limit(maxforce);  // Limit to maximum steering force
    
    acc.add(steer);
    
  } 
  
  
  // A method that calculates a steering vector towards a target
  // Takes a second argument, if true, it slows down as it approaches the target
  public PVector steer(PVector target, boolean slowdown) {
    PVector steer;  // The steering vector
    PVector desired = PVector.sub(target,loc);  // A vector pointing from the location to the target
    float d = desired.mag(); // Distance from the target is the magnitude of the vector
    // If the distance is greater than 0, calc steering (otherwise return zero vector)
    if (d > 0) {
      // Normalize desired
      desired.normalize();
      // Two options for desired vector magnitude (1 -- based on distance, 2 -- maxspeed)
      if ((slowdown) && (d < 100.0f)) desired.mult(maxspeed*(d/100.0f)); // This damping is somewhat arbitrary
      else desired.mult(maxspeed);
      // Steering = Desired minus Velocity
      steer = PVector.sub(desired,vel);
      steer.limit(maxforce);  // Limit to maximum steering force
    } else {
      steer = new PVector(0,0);
    }
    return steer;
  }
  
  public void render(boolean is_init) {
    // Display dandy image the direction of velocity
    if(at_init == false){
      pushMatrix();
      translate(loc.x,loc.y);
      rotate(PI);
      image(dandy, 0, 0, sizemodx, sizemody);
      popMatrix();
    }
    else{
      pushMatrix();
      translate(loc.x, loc.y);
      rotate(init_angle+radians(270));
      translate(0-(sizemodx/2), 0);
      image(dandy, 0, 0, sizemodx, sizemody);
      popMatrix();
    }
  }
  
  // Wraparound
  public void borders() {
    if (loc.x < -r) loc.x = width+r;
    if (loc.x > width+r) loc.x = -r;
    if (loc.y > height){
      vel = new PVector(0,0);
      loc.y = height;
    }
  }
  
    // Separation
  public PVector separate (ArrayList dandys) {
    float desiredseparation = r;
    PVector steer = new PVector(0,0,0);
    int count = 0;
    // For every Dandy in the system, check if it's too close
    for (int i = 0 ; i < dandys.size(); i++) {
      Dandy other = (Dandy) dandys.get(i);
      float d = PVector.dist(loc,other.loc);
      // If the distance is greater than 0 and less than an arbitrary amount (0 when you are yourself)
      if ((d > 0) && (d < desiredseparation)) {
        // Calculate vector pointing away from neighbor
        PVector diff = PVector.sub(loc,other.loc);
        diff.normalize();
        diff.div(d);        // Weight by distance
        steer.add(diff);
        count++;            // Keep track of how many
      }
    }
    // Average -- divide by how many
    if (count > 0) {
      steer.div((float)count);
    }

    // As long as the vector is greater than 0
    if (steer.mag() > 0) {
      // Implement Reynolds: Steering = Desired - Velocity
      steer.normalize();
      steer.mult(maxspeed);
      steer.sub(vel);
      steer.limit(maxforce);
    }
    return steer;
  }
  
  public boolean at_init(){
    return at_init;
  }
}
class Stem{ 
  public void display(){
    stroke(255);
    strokeWeight(10);
    noFill();
    bezier(width/2, height/2, width/2+105, height/2+85, width/2+105, height/2+150, width/2, height); 
    
    fill(255);
    noStroke();
    smooth();
    ellipse(width/2, height/2, 70, 70); 
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "proj3" });
  }
}
