import krister.Ess.*;

ArrayList dandys;
Stem stem;

/* Audio Set-up */
int bufferSize;
int steps;
float limitDiff;
int numAverages=32;
float myDamp=.1f;
float maxLimit,minLimit;
FFT myFFT;
AudioInput myInput;
PVector[] wind;

void setup(){
  size(900,600);
  smooth();
  
  dandys = new ArrayList();
  
  float angle;
  float x;
  float y;
  for(float i= 0; i < 126; i ++){
    angle = (float)i/126*TWO_PI;
    x = width/2 + 26*cos(angle);
    y = height/2 + 26*sin(angle);
    dandys.add(new Dandy(new PVector(x,y), angle, ((int)i)%7, 5.0, 1.0));
  }
  
  stem = new Stem();
  wind = new PVector[32];
  
  
  // start up Ess
  Ess.start(this);  

  // set up our AudioInput
  bufferSize=32;
  myInput=new AudioInput(bufferSize);

  // set up our FFT
  myFFT=new FFT(bufferSize*2);
  myFFT.equalizer(true);

  // set up our FFT normalization/dampening
  minLimit=.005;
  maxLimit=.05;
  myFFT.limits(minLimit,maxLimit);
  myFFT.damp(myDamp);
  myFFT.averages(numAverages);

  // get the number of bins per average 
  steps=bufferSize/numAverages;

  // get the distance of travel between minimum and maximum limits
  limitDiff=maxLimit-minLimit;

  myInput.start();
  frameRate(45);
}

void draw(){
  background(#97C9FF);
  
  fill(#8DBFFA);
  /* Draw audio bars in background*/
  for (int i=0; i<32; i++) {
     if(i == 0){
       rect(28*i,height,28,-(myFFT.spectrum[1]*1000));
       continue;
     }
     if(i == 31){       
       rect(28*i,height,32,-(myFFT.spectrum[1]*1000));
       break;
     }
       
     rect(28*i,height,28,-(myFFT.spectrum[i]*1000));
  }
  
  stem.display();
  updateWind();
  for(int i=0; i < 126; i++){
    Dandy index = (Dandy)dandys.get(i);
    index.run(dandys, wind);
  }
}
  
// Make a new flowfield
 void keyPressed() {
  for(int i=0; i < 126; i++){
    Dandy index = (Dandy)dandys.get(i);
    
    if(index.at_init()){
      break;
    }
    index.wind();
  }
}

public void audioInputData(AudioInput theInput) {
  myFFT.getSpectrum(myInput);
}

void updateWind() {
  float sizemod = 0;
  float theta = 0;

  for (int i = 0; i < 32; i++) {
    // Use perlin noise to get an angle between 0 and 2 PI
    theta = map(myFFT.spectrum[i],.005,.05,0,TWO_PI);
    
    // Generate length of the vector to indicate the level of sound
    sizemod = 1000*myFFT.spectrum[i];
    
    // Polar to cartesian coordinate transformation to get x and y components of the vector
    wind[i] = new PVector(sizemod*cos(theta),sizemod*sin(theta));
  }
}

